@@ -5,10 +5,12 @@
 import sys
 import os
 
+from zope.interface import implements
+
 from twisted.internet.protocol import Factory
 from twisted.protocols.basic import LineOnlyReceiver
 from twisted.internet.error import CannotListenError, ConnectionDone
-from twisted.internet import reactor
+from twisted.internet import reactor, interfaces
 
 from gnutls.constants import *
 from gnutls.crypto import *
@@ -17,17 +19,40 @@ from gnutls.interfaces.twisted import X509Credentials
 
 class EchoProtocol(LineOnlyReceiver):
 
+    implements(interfaces.IHalfCloseableProtocol)
+
     def connectionMade(self):
-        print "connection established"
+        session = self.transport.socket
+        try:
+            peer_name = session.peer_certificate.subject
+        except AttributeError:
+            peer_name = 'Unknown'
+        print '\nNew connection from:', peer_name
+        print 'Protocol:     ', session.protocol
+        print 'KX algorithm: ', session.kx_algorithm
+        print 'Cipher:       ', session.cipher
+        print 'MAC algorithm:', session.mac_algorithm
+        print 'Compression:  ', session.compression
+        self.transport.loseWriteConnection()
+        reactor.callLater(5, self.transport.loseConnection)
 
     def lineReceived(self, line):
-        print "received", line
+        if line == 'quit':
+            self.transport.loseConnection()
+            return
+        print "received:", line
+        #self.sendLine(line)
 
     def connectionLost(self, reason):
-        if reason.type != ConnectionDone:
+        #if reason.type != ConnectionDone:
             print "Connection was lost:", str(reason.value)
-        else:
-            print "Connection done"
+
+    def readConnectionLost(self):
+        print "read connection was lost"
+
+    def writeConnectionLost(self):
+        print "write connection was lost"
+
 
 class EchoFactory(Factory):
     protocol = EchoProtocol
